@@ -1,4 +1,4 @@
-/* Copyright 2012--2016 The Tor Project
+/* Copyright 2012--2017 The Tor Project
  * See LICENSE for licensing information */
 
 package org.torproject.descriptor.impl;
@@ -312,6 +312,11 @@ public class DirectoryKeyCertificateImpl extends DescriptorImpl
 
   @Override
   public String getCertificateDigest() {
+    return this.getDigestSha1Hex();
+  }
+
+  @Override
+  public String getDigestSha1Hex() {
     return this.certificateDigest;
   }
 }
