@@ -66,6 +66,7 @@ public abstract class ExtraInfoDescriptorImpl extends DescriptorImpl
     atMostOnceKeywords.addAll(connBiDirectStatsKeywords);
     atMostOnceKeywords.addAll(exitStatsKeywords);
     atMostOnceKeywords.addAll(bridgeStatsKeywords);
+    atMostOnceKeywords.add("padding-counts");
     this.checkAtMostOnceKeywords(atMostOnceKeywords);
     this.checkKeywordsDependOn(dirreqStatsKeywords, "dirreq-stats-end");
     this.checkKeywordsDependOn(entryStatsKeywords, "entry-stats-end");
@@ -220,6 +221,9 @@ public abstract class ExtraInfoDescriptorImpl extends DescriptorImpl
         case "hidserv-dir-onions-seen":
           this.parseHidservDirOnionsSeenLine(line, lineNoOpt, partsNoOpt);
           break;
+        case "padding-counts":
+          this.parsePaddingCountsLine(line, lineNoOpt, partsNoOpt);
+          break;
         case "identity-ed25519":
           this.parseIdentityEd25519Line(line, lineNoOpt, partsNoOpt);
           nextCrypto = "identity-ed25519";
@@ -270,9 +274,9 @@ public abstract class ExtraInfoDescriptorImpl extends DescriptorImpl
                 }
                 this.unrecognizedLines.addAll(cryptoLines);
               }
-              cryptoLines = null;
-              nextCrypto = "";
           }
+          cryptoLines = null;
+          nextCrypto = "";
           break;
         default:
           if (cryptoLines != null) {
@@ -324,7 +328,7 @@ public abstract class ExtraInfoDescriptorImpl extends DescriptorImpl
 
   private void parseGeoipDbDigestLine(String line, String lineNoOpt,
       String[] partsNoOpt) throws DescriptorParseException {
-    if (partsNoOpt.length != 2) {
+    if (partsNoOpt.length < 2) {
       throw new DescriptorParseException("Illegal line '" + line
           + "' in extra-info descriptor.");
     }
@@ -334,7 +338,7 @@ public abstract class ExtraInfoDescriptorImpl extends DescriptorImpl
 
   private void parseGeoip6DbDigestLine(String line, String lineNoOpt,
       String[] partsNoOpt) throws DescriptorParseException {
-    if (partsNoOpt.length != 2) {
+    if (partsNoOpt.length < 2) {
       throw new DescriptorParseException("Illegal line '" + line
           + "' in extra-info descriptor.");
     }
@@ -344,7 +348,7 @@ public abstract class ExtraInfoDescriptorImpl extends DescriptorImpl
 
   private void parseGeoipStartTimeLine(String line, String lineNoOpt,
       String[] partsNoOpt) throws DescriptorParseException {
-    if (partsNoOpt.length != 3) {
+    if (partsNoOpt.length < 3) {
       throw new DescriptorParseException("Illegal line '" + line
           + "' in extra-info descriptor.");
     }
@@ -369,7 +373,7 @@ public abstract class ExtraInfoDescriptorImpl extends DescriptorImpl
 
   private long[] parseStatsEndLine(String line, String[] partsNoOpt,
       int partsNoOptExpectedLength) throws DescriptorParseException {
-    if (partsNoOpt.length != partsNoOptExpectedLength
+    if (partsNoOpt.length < partsNoOptExpectedLength
         || partsNoOpt[3].length() < 2 || !partsNoOpt[3].startsWith("(")
         || !partsNoOpt[4].equals("s)")) {
       throw new DescriptorParseException("Illegal line '" + line + "'.");
@@ -424,7 +428,7 @@ public abstract class ExtraInfoDescriptorImpl extends DescriptorImpl
   private double parseShareLine(String line, String[] partsNoOpt)
       throws DescriptorParseException {
     double share = -1.0;
-    if (partsNoOpt.length == 2 && partsNoOpt[1].length() >= 2
+    if (partsNoOpt.length >= 2 && partsNoOpt[1].length() >= 2
         && partsNoOpt[1].endsWith("%")) {
       String shareString = partsNoOpt[1];
       shareString = shareString.substring(0, shareString.length() - 1);
@@ -550,7 +554,7 @@ public abstract class ExtraInfoDescriptorImpl extends DescriptorImpl
       String lineNoOpt, String[] partsNoOpt)
       throws DescriptorParseException {
     int circuits = -1;
-    if (partsNoOpt.length == 2) {
+    if (partsNoOpt.length >= 2) {
       try {
         circuits = Integer.parseInt(partsNoOpt[1]);
       } catch (NumberFormatException e) {
@@ -756,6 +760,16 @@ public abstract class ExtraInfoDescriptorImpl extends DescriptorImpl
         partsNoOpt, 2);
   }
 
+  private void parsePaddingCountsLine(String line, String lineNoOpt,
+      String[] partsNoOpt) throws DescriptorParseException {
+    long[] parsedStatsEndData = this.parseStatsEndLine(line, partsNoOpt,
+        6);
+    this.paddingCountsStatsEndMillis = parsedStatsEndData[0];
+    this.paddingCountsStatsIntervalLength = parsedStatsEndData[1];
+    this.paddingCounts = ParseHelper.parseSpaceSeparatedStringKeyLongValueMap(
+        line, partsNoOpt, 5);
+  }
+
   private void parseRouterSignatureLine(String line, String lineNoOpt,
       String[] partsNoOpt) throws DescriptorParseException {
     if (!lineNoOpt.equals("router-signature")) {
@@ -889,6 +903,11 @@ public abstract class ExtraInfoDescriptorImpl extends DescriptorImpl
 
   @Override
   public String getExtraInfoDigest() {
+    return this.getDigestSha1Hex();
+  }
+
+  @Override
+  public String getDigestSha1Hex() {
     return this.extraInfoDigest;
   }
 
@@ -896,6 +915,11 @@ public abstract class ExtraInfoDescriptorImpl extends DescriptorImpl
 
   @Override
   public String getExtraInfoDigestSha256() {
+    return this.getDigestSha256Base64();
+  }
+
+  @Override
+  public String getDigestSha256Base64() {
     return this.extraInfoDigestSha256;
   }
 
@@ -938,6 +962,11 @@ public abstract class ExtraInfoDescriptorImpl extends DescriptorImpl
 
   @Override
   public String getGeoipDbDigest() {
+    return this.getGeoipDbDigestSha1Hex();
+  }
+
+  @Override
+  public String getGeoipDbDigestSha1Hex() {
     return this.geoipDbDigest;
   }
 
@@ -945,6 +974,11 @@ public abstract class ExtraInfoDescriptorImpl extends DescriptorImpl
 
   @Override
   public String getGeoip6DbDigest() {
+    return this.getGeoip6DbDigestSha1Hex();
+  }
+
+  @Override
+  public String getGeoip6DbDigestSha1Hex() {
     return this.geoip6DbDigest;
   }
 
@@ -1321,6 +1355,28 @@ public abstract class ExtraInfoDescriptorImpl extends DescriptorImpl
         : new HashMap<>(this.hidservDirOnionsSeenParameters);
   }
 
+  private long paddingCountsStatsEndMillis = -1L;
+
+  @Override
+  public long getPaddingCountsStatsEndMillis() {
+    return this.paddingCountsStatsEndMillis;
+  }
+
+  private long paddingCountsStatsIntervalLength = -1L;
+
+  @Override
+  public long getPaddingCountsStatsIntervalLength() {
+    return this.paddingCountsStatsIntervalLength;
+  }
+
+  private Map<String, Long> paddingCounts;
+
+  @Override
+  public Map<String, Long> getPaddingCounts() {
+    return this.paddingCounts == null ? null
+        : new HashMap<>(this.paddingCounts);
+  }
+
   private String routerSignature;
 
   @Override
