@@ -13,7 +13,10 @@ import org.torproject.descriptor.DescriptorParseException;
 import org.torproject.descriptor.ExtraInfoDescriptor;
 import org.torproject.descriptor.RelayExtraInfoDescriptor;
 
+import org.hamcrest.Matchers;
+import org.junit.Rule;
 import org.junit.Test;
+import org.junit.rules.ExpectedException;
 
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
@@ -25,6 +28,9 @@ import java.util.SortedMap;
 /* Test parsing of extra-info descriptors. */
 public class ExtraInfoDescriptorImplTest {
 
+  @Rule
+  public ExpectedException thrown = ExpectedException.none();
+
   /* Helper class to build a descriptor based on default data and
    * modifications requested by test methods. */
   private static class DescriptorBuilder {
@@ -180,6 +186,15 @@ public class ExtraInfoDescriptorImplTest {
       return new RelayExtraInfoDescriptorImpl(db.buildDescriptor(), true);
     }
 
+    private String paddingCountsLine = null;
+
+    private static ExtraInfoDescriptor createWithPaddingCountsLine(
+        String line) throws DescriptorParseException {
+      DescriptorBuilder db = new DescriptorBuilder();
+      db.paddingCountsLine = line;
+      return new RelayExtraInfoDescriptorImpl(db.buildDescriptor(), true);
+    }
+
     private String unrecognizedLine = null;
 
     private static ExtraInfoDescriptor createWithUnrecognizedLine(
@@ -288,6 +303,9 @@ public class ExtraInfoDescriptorImplTest {
       if (this.hidservStatsLines != null) {
         sb.append(this.hidservStatsLines).append("\n");
       }
+      if (this.paddingCountsLine != null) {
+        sb.append(this.paddingCountsLine).append("\n");
+      }
       if (this.unrecognizedLine != null) {
         sb.append(this.unrecognizedLine).append("\n");
       }
@@ -1093,6 +1111,14 @@ public class ExtraInfoDescriptorImplTest {
         + "81281024,,60625920,67922944");
   }
 
+  @Test()
+  public void testDirreqWriteHistoryExtraArg() throws DescriptorParseException {
+    DescriptorBuilder.createWithDirreqWriteHistoryLine(
+        "dirreq-write-history "
+        + "2012-02-11 09:03:39 (900 s) 81281024,64996352,60625920,"
+        + "67922944 bin_size=1024");
+  }
+
   @Test(expected = DescriptorParseException.class)
   public void testDirreqReadHistoryMissingBytesEnd()
       throws DescriptorParseException {
@@ -1107,7 +1133,7 @@ public class ExtraInfoDescriptorImplTest {
         .createWithGeoipDbDigestLine("geoip-db-digest "
         + "916A3CA8B7DF61473D5AE5B21711F35F301CE9E8");
     assertEquals("916A3CA8B7DF61473D5AE5B21711F35F301CE9E8",
-        descriptor.getGeoipDbDigest());
+        descriptor.getGeoipDbDigestSha1Hex());
   }
 
   @Test(expected = DescriptorParseException.class)
@@ -1130,11 +1156,33 @@ public class ExtraInfoDescriptorImplTest {
     DescriptorBuilder.createWithGeoipDbDigestLine("geoip-db-digest");
   }
 
+  @Test()
+  public void testGeoipDbDigestExtraArg() throws DescriptorParseException {
+    ExtraInfoDescriptor descriptor = DescriptorBuilder
+        .createWithGeoip6DbDigestLine("geoip-db-digest "
+            + "916A3CA8B7DF61473D5AE5B21711F35F301CE9E8 "
+            + "yblgrXtEgF3glaKv5ZvHhRREUI1t1c37SxparXSmYR4Q1yiK5zg4HE8eT9ILPRW9"
+            + "3I5W/pZGQxL8Bu42dGjnAQ");
+    assertEquals("916A3CA8B7DF61473D5AE5B21711F35F301CE9E8",
+        descriptor.getGeoipDbDigest());
+  }
+
   @Test()
   public void testGeoip6DbDigestValid() throws DescriptorParseException {
     ExtraInfoDescriptor descriptor = DescriptorBuilder
         .createWithGeoip6DbDigestLine("geoip6-db-digest "
         + "916A3CA8B7DF61473D5AE5B21711F35F301CE9E8");
+    assertEquals("916A3CA8B7DF61473D5AE5B21711F35F301CE9E8",
+        descriptor.getGeoip6DbDigestSha1Hex());
+  }
+
+  @Test()
+  public void testGeoip6DbDigestExtraArg() throws DescriptorParseException {
+    ExtraInfoDescriptor descriptor = DescriptorBuilder
+        .createWithGeoip6DbDigestLine("geoip6-db-digest "
+            + "916A3CA8B7DF61473D5AE5B21711F35F301CE9E8 "
+            + "yblgrXtEgF3glaKv5ZvHhRREUI1t1c37SxparXSmYR4Q1yiK5zg4HE8eT9ILPRW9"
+            + "3I5W/pZGQxL8Bu42dGjnAQ");
     assertEquals("916A3CA8B7DF61473D5AE5B21711F35F301CE9E8",
         descriptor.getGeoip6DbDigest());
   }
@@ -1241,6 +1289,14 @@ public class ExtraInfoDescriptorImplTest {
         + "ru=352,fr=208,gb=208,ir=200");
   }
 
+  @Test()
+  public void testGeoipClientOriginsExtraArg()
+      throws DescriptorParseException {
+    GeoipStatsBuilder.createWithGeoipClientOriginsLine(
+        "geoip-client-origins de=1152,de=952,cn=896,us=712,it=504 "
+            + "ru=352 fr=208 gb=208 ir=200");
+  }
+
   @Test()
   public void testDirreqStatsValid() throws DescriptorParseException {
     ExtraInfoDescriptor descriptor = DirreqStatsBuilder
@@ -1279,6 +1335,13 @@ public class ExtraInfoDescriptorImplTest {
         + "2012-02-11 00:59:53 (172800 s)");
   }
 
+  @Test()
+  public void testDirreqStatsExtraArg()
+      throws DescriptorParseException {
+    DirreqStatsBuilder.createWithDirreqStatsEndLine("dirreq-stats-end "
+        + "2012-02-11 00:59:53 (172800 s) XXXXXXXXXXXXXXXXXXXXXXXXXXX");
+  }
+
   @Test(expected = DescriptorParseException.class)
   public void testDirreqV3IpsThreeLetterCountry()
       throws DescriptorParseException {
@@ -1286,6 +1349,13 @@ public class ExtraInfoDescriptorImplTest {
         + "usa=1544");
   }
 
+  @Test()
+  public void testDirreqV3IpsExtraArg()
+      throws DescriptorParseException {
+    DirreqStatsBuilder.createWithDirreqV3IpsLine("dirreq-v3-ips "
+        + "ab=12,cd=34 ef=56");
+  }
+
   @Test()
   public void testDirreqV2IpsDigitCountry()
       throws DescriptorParseException {
@@ -1312,6 +1382,22 @@ public class ExtraInfoDescriptorImplTest {
         + "ok==10848");
   }
 
+  @Test()
+  public void testDirreqV3RespEmptyString()
+      throws DescriptorParseException {
+    this.thrown.expect(DescriptorParseException.class);
+    this.thrown.expectMessage(
+        "Line 'dirreq-v3-resp =10848' contains an illegal key or value.");
+    DirreqStatsBuilder.createWithDirreqV3RespLine("dirreq-v3-resp =10848");
+  }
+
+  @Test()
+  public void testDirreqV3RespExtraArg()
+      throws DescriptorParseException {
+    DirreqStatsBuilder.createWithDirreqV3RespLine("dirreq-v3-resp "
+        + "ok=1084 4801=ko");
+  }
+
   @Test(expected = DescriptorParseException.class)
   public void testDirreqV2RespNull()
       throws DescriptorParseException {
@@ -1333,6 +1419,13 @@ public class ExtraInfoDescriptorImplTest {
         + "0.37");
   }
 
+  @Test()
+  public void testDirreqV3ShareExtraArg()
+      throws DescriptorParseException {
+    DirreqStatsBuilder.createWithDirreqV3ShareLine("dirreq-v3-share "
+        + "0.37% 123456");
+  }
+
   @Test(expected = DescriptorParseException.class)
   public void testDirreqV3DirectDlSpace()
       throws DescriptorParseException {
@@ -1361,6 +1454,13 @@ public class ExtraInfoDescriptorImplTest {
         "dirreq-v2-tunneled-dl complete=0.001");
   }
 
+  @Test()
+  public void testDirreqV3TunneledDlExtraArg()
+      throws DescriptorParseException {
+    DirreqStatsBuilder.createWithDirreqV2TunneledDlLine(
+        "dirreq-v2-tunneled-dl complete=-8 incomplete=1/-8");
+  }
+
   @Test()
   public void testEntryStatsValid() throws DescriptorParseException {
     ExtraInfoDescriptor descriptor = EntryStatsBuilder
@@ -1442,6 +1542,13 @@ public class ExtraInfoDescriptorImplTest {
         "cell-circuits-per-decile -866");
   }
 
+  @Test()
+  public void testCellCircuitsPerDecileExtraArg()
+      throws DescriptorParseException {
+    CellStatsBuilder.createWithCellCircuitsPerDecileLine(
+        "cell-circuits-per-decile 866 866 866 866 866");
+  }
+
   @Test()
   public void testConnBiDirectValid()
       throws DescriptorParseException {
@@ -1464,6 +1571,13 @@ public class ExtraInfoDescriptorImplTest {
         + "2012-02-11 01:59:39 (86400 s) 42173,1591,1310,1744,42");
   }
 
+  @Test()
+  public void testConnBiDirectStatsExtraArg()
+      throws DescriptorParseException {
+    DescriptorBuilder.createWithConnBiDirectLine("conn-bi-direct "
+        + "2012-02-11 01:59:39 (86400 s) 42173,1591,1310,1744 +1");
+  }
+
   @Test()
   public void testExitStatsValid() throws DescriptorParseException {
     ExtraInfoDescriptor descriptor = ExitStatsBuilder
@@ -1519,6 +1633,16 @@ public class ExtraInfoDescriptorImplTest {
         "exit-kibibytes-written unknown=74647");
   }
 
+  @Test()
+  public void testExitStatsWrittenEmptyString()
+      throws DescriptorParseException {
+    this.thrown.expect(DescriptorParseException.class);
+    this.thrown.expectMessage("Line 'exit-kibibytes-written =74647' contains "
+        + "an illegal key or value in list element '=74647'.");
+    ExitStatsBuilder.createWithExitKibibytesWrittenLine(
+        "exit-kibibytes-written =74647");
+  }
+
   @Test(expected = DescriptorParseException.class)
   public void testExitStatsReadNegativeBytes()
       throws DescriptorParseException {
@@ -1540,6 +1664,12 @@ public class ExtraInfoDescriptorImplTest {
         "exit-streams-opened 25=2147483648");
   }
 
+  @Test()
+  public void testExitStatsStreamsExtraArg() throws DescriptorParseException {
+    ExitStatsBuilder.createWithExitStreamsOpenedLine(
+        "exit-streams-opened 25=21474 3648");
+  }
+
   @Test()
   public void testBridgeStatsValid() throws DescriptorParseException {
     ExtraInfoDescriptor descriptor = BridgeStatsBuilder
@@ -1570,6 +1700,13 @@ public class ExtraInfoDescriptorImplTest {
         + "2012-02-11 01:59:39 (0 s)");
   }
 
+  @Test()
+  public void testBridgeStatsEndExtraArg()
+      throws DescriptorParseException {
+    BridgeStatsBuilder.createWithBridgeStatsEndLine("bridge-stats-end "
+        + "2012-02-11 01:59:39 (86400 s) 99999999999999999999999999999999");
+  }
+
   @Test(expected = DescriptorParseException.class)
   public void testBridgeIpsDouble()
       throws DescriptorParseException {
@@ -1585,6 +1722,12 @@ public class ExtraInfoDescriptorImplTest {
         0x69, 0x70, 0x73 }, false);               // "ips" (no newline)
   }
 
+  @Test()
+  public void testBridgeIpsExtraArg()
+      throws DescriptorParseException {
+    BridgeStatsBuilder.createWithBridgeIpsLine("bridge-ips ir=24 5");
+  }
+
   @Test(expected = DescriptorParseException.class)
   public void testBridgeIpVersionsDouble()
       throws DescriptorParseException {
@@ -1592,6 +1735,13 @@ public class ExtraInfoDescriptorImplTest {
         "bridge-ip-versions v4=24.5");
   }
 
+  @Test()
+  public void testBridgeIpVersionsExtraArg()
+      throws DescriptorParseException {
+    BridgeStatsBuilder.createWithBridgeIpVersionsLine(
+        "bridge-ip-versions v4=24 5");
+  }
+
   @Test(expected = DescriptorParseException.class)
   public void testBridgeIpTransportsDouble()
       throws DescriptorParseException {
@@ -1606,6 +1756,143 @@ public class ExtraInfoDescriptorImplTest {
         "bridge-ip-transports meek=32,obfs3_websocket=8,websocket=64");
   }
 
+  @Test()
+  public void testBridgeIpTransportsExtraArg()
+      throws DescriptorParseException {
+    BridgeStatsBuilder.createWithBridgeIpTransportsLine(
+        "bridge-ip-transports obfs2=24 5");
+  }
+
+  @Test()
+  public void testPaddingCountsValid()
+      throws DescriptorParseException {
+    ExtraInfoDescriptor descriptor = DescriptorBuilder
+        .createWithPaddingCountsLine("padding-counts 2017-05-10 01:48:43 "
+        + "(86400 s) bin-size=10000 write-drop=10000 write-pad=10000 "
+        + "write-total=10000 read-drop=10000 read-pad=10000 read-total=70000 "
+        + "enabled-read-pad=0 enabled-read-total=0 enabled-write-pad=0 "
+        + "enabled-write-total=0 max-chanpad-timers=0");
+    checkPaddingCounts(descriptor, new String[] { "bin-size", "write-drop",
+        "write-pad", "write-total", "read-drop", "read-pad", "read-total",
+        "enabled-read-pad", "enabled-read-total", "enabled-write-pad",
+        "enabled-write-total", "max-chanpad-timers"},
+        new long[] { 10000L, 10000L, 10000L, 10000L, 10000L, 10000L, 70000L,
+            0L, 0L, 0L, 0L, 0L});
+  }
+
+  @Test()
+  public void testPaddingOtherCountsValid()
+      throws DescriptorParseException {
+    ExtraInfoDescriptor descriptor = DescriptorBuilder
+        .createWithPaddingCountsLine("padding-counts 2017-05-10 01:48:43 "
+        + "(86400 s) xbin-size=10000 xwrite-drop=10000 xwrite-pad=10000 "
+        + "write-total=10000 xread-drop=10000 xread-pad=10000 read-total=70000 "
+        + "enabled-read-pad=0 enabled-read-total=0 xenabled-write-pad=0 "
+        + "enabled-write-total=0");
+    checkPaddingCounts(descriptor, new String[] { "xbin-size", "xwrite-drop",
+        "xwrite-pad", "write-total", "xread-drop", "xread-pad", "read-total",
+        "enabled-read-pad", "enabled-read-total", "xenabled-write-pad",
+        "enabled-write-total"},
+        new long[] { 10000L, 10000L, 10000L, 10000L, 10000L, 10000L, 70000L,
+            0L, 0L, 0L, 0L});
+  }
+
+  private void checkPaddingCounts(ExtraInfoDescriptor descriptor,
+      String[] keys, long[] vals) {
+    assertEquals(1494380923000L,
+        descriptor.getPaddingCountsStatsEndMillis());
+    assertEquals(86400, descriptor.getPaddingCountsStatsIntervalLength());
+    for (int k = 0; k < keys.length; k++) {
+      assertEquals(vals[k],
+          (long) descriptor.getPaddingCounts().get(keys[k]));
+    }
+  }
+
+  @Test()
+  public void testPaddingCountsValidFutureProof()
+      throws DescriptorParseException {
+    ExtraInfoDescriptor descriptor = DescriptorBuilder
+        .createWithPaddingCountsLine("padding-counts 2017-05-10 01:48:43 "
+        + "(86400 s) write-drop=10000 write-pad=10000 bin-size=10000 "
+        + "write-total=10000 read-drop=10000 read-pad=10000 read-total=70000 "
+        + "enabled-read-pad=0 enabled-read-total=0 enabled-write-pad=0 "
+        + "enabled-write-total=0 max-chanpad-timers=0 some-new-value=42");
+    checkPaddingCounts(descriptor, new String[] { "bin-size", "write-drop",
+        "write-pad", "write-total", "read-drop", "read-pad", "read-total",
+        "enabled-read-pad", "enabled-read-total", "enabled-write-pad",
+        "enabled-write-total", "max-chanpad-timers", "some-new-value"},
+        new long[] { 10000L, 10000L, 10000L, 10000L, 10000L, 10000L, 70000L,
+            0L, 0L, 0L, 0L, 0L, 42L});
+  }
+
+  @Test(expected = DescriptorParseException.class)
+  public void testPaddingCountsNoTime() throws DescriptorParseException {
+    DescriptorBuilder.createWithPaddingCountsLine("padding-counts 2017-05-10 "
+        + "(86400 s) bin-size=10000 write-drop=10000");
+  }
+
+  @Test()
+  public void testPaddingCountsNoInterval() throws DescriptorParseException {
+    this.thrown.expect(DescriptorParseException.class);
+    this.thrown.expectMessage(Matchers
+        .allOf(Matchers.containsString("Illegal line"), Matchers
+        .containsString("'padding-counts 2017-05-10 01:48:43 "
+        + "bin-size=10000 write-drop=10000'")));
+    DescriptorBuilder.createWithPaddingCountsLine("padding-counts 2017-05-10 "
+        + "01:48:43 bin-size=10000 write-drop=10000");
+  }
+
+  @Test()
+  public void testPaddingCountsNoKey() throws DescriptorParseException {
+    this.thrown.expect(DescriptorParseException.class);
+    this.thrown.expectMessage(Matchers
+        .allOf(Matchers.containsString("illegal key or value in list element"),
+        Matchers.containsString("=7")));
+    DescriptorBuilder.createWithPaddingCountsLine("padding-counts 2017-05-10 "
+        + "01:48:43 (86400 s) write-total=9 write-drop=10000 =7 x=8");
+  }
+
+  @Test()
+  public void testPaddingCountsNoValue() throws DescriptorParseException {
+    this.thrown.expect(DescriptorParseException.class);
+    this.thrown.expectMessage(Matchers
+        .allOf(Matchers.containsString("illegal key or value in list element"),
+        Matchers.containsString("'write-drop='")));
+    DescriptorBuilder.createWithPaddingCountsLine("padding-counts 2017-05-10 "
+        + "01:48:43 (86400 s) write-total=7 write-drop= bin-size=10000 ");
+  }
+
+  @Test()
+  public void testPaddingCountsKeyRepeated() throws DescriptorParseException {
+    this.thrown.expect(DescriptorParseException.class);
+    this.thrown.expectMessage(Matchers
+        .allOf(Matchers.containsString("contains an already defined key"),
+        Matchers.containsString("'a'")));
+    DescriptorBuilder.createWithPaddingCountsLine("padding-counts 2017-05-10 "
+        + "01:48:43 (86400 s) a=1 b=2 a=3 b=4");
+  }
+
+  @Test()
+  public void testPaddingCountsCommaSeparatedList()
+      throws DescriptorParseException {
+    this.thrown.expect(DescriptorParseException.class);
+    this.thrown.expectMessage(Matchers
+        .allOf(Matchers.containsString("Line 'padding-counts 2017-05-10 "),
+        Matchers.containsString("'bin-size=10000,write-drop=10000'")));
+    DescriptorBuilder.createWithPaddingCountsLine("padding-counts 2017-05-10 "
+        + "01:48:43 (86400 s) bin-size=10000,write-drop=10000");
+  }
+
+  @Test()
+  public void testPaddingCountsNoList() throws DescriptorParseException {
+    this.thrown.expect(DescriptorParseException.class);
+    this.thrown.expectMessage(Matchers
+        .allOf(Matchers.containsString("Illegal line"),
+        Matchers.containsString("'padding-counts 2017-05-10 (86400 s)'")));
+    DescriptorBuilder.createWithPaddingCountsLine("padding-counts 2017-05-10 "
+        + "(86400 s)");
+  }
+
   @Test()
   public void testHidservStatsValid() throws DescriptorParseException {
     ExtraInfoDescriptor descriptor = HidservStatsBuilder
@@ -1669,6 +1956,13 @@ public class ExtraInfoDescriptorImplTest {
         "hidserv-dir-onions-seen -3 delta_f=A epsilon=B bin_size=C");
   }
 
+  @Test()
+  public void testHidservDirOnionsSeenExtraArg()
+      throws DescriptorParseException {
+    HidservStatsBuilder.createWithHidservDirOnionsSeenLine(
+        "hidserv-dir-onions-seen -3 delta_f=8 epsilon=0.30 bin_size=8 pi=3");
+  }
+
   @Test()
   public void testRouterSignatureOpt()
       throws DescriptorParseException {
@@ -1805,6 +2099,14 @@ public class ExtraInfoDescriptorImplTest {
         + ROUTER_SIG_ED25519_LINE);
   }
 
+  @Test(expected = DescriptorParseException.class)
+  public void testEd25519FollowedbyUnrecognizedLine()
+      throws DescriptorParseException {
+    DescriptorBuilder.createWithEd25519Lines(IDENTITY_ED25519_LINES,
+        MASTER_KEY_ED25519_LINE, ROUTER_SIG_ED25519_LINE
+        + "\nunrecognized-line 1");
+  }
+
   @Test()
   public void testExtraInfoDigestSha256Relay()
       throws DescriptorParseException {
@@ -1834,7 +2136,7 @@ public class ExtraInfoDescriptorImplTest {
     RelayExtraInfoDescriptor descriptor =
         new RelayExtraInfoDescriptorImpl(descriptorBytes, true);
     assertEquals("Pt1BtzfRwhYqGCDo8jjchS8nJP3ovrDyHGn+dqPbMgw",
-        descriptor.getExtraInfoDigestSha256());
+        descriptor.getDigestSha256Base64());
   }
 
   @Test()
@@ -1858,7 +2160,7 @@ public class ExtraInfoDescriptorImplTest {
     BridgeExtraInfoDescriptor descriptor =
         new BridgeExtraInfoDescriptorImpl(descriptorBytes, true);
     assertEquals("TvrqpjI7OmCtwGwair/NHUxg5ROVVQYz6/EDyXsDHR4",
-        descriptor.getExtraInfoDigestSha256());
+        descriptor.getDigestSha256Base64());
   }
 }
 
