@@ -1,4 +1,4 @@
-/* Copyright 2016 The Tor Project
+/* Copyright 2016--2017 The Tor Project
  * See LICENSE for licensing information */
 
 package org.torproject.descriptor.index;
@@ -26,6 +26,13 @@ public class DirectoryNode implements Comparable<DirectoryNode> {
   @Expose
   public final SortedSet<DirectoryNode> directories;
 
+  /* Added to satisfy Gson. */
+  private DirectoryNode() {
+    path = null;
+    files = null;
+    directories = null;
+  }
+
   /** A directory for the JSON structure. */
   public DirectoryNode(String path, SortedSet<FileNode> files,
                 SortedSet<DirectoryNode> directories) {
